"""empty message

Revision ID: af1790868e2c
Revises: a519c710877c
Create Date: 2019-02-26 20:36:21.400109

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "af1790868e2c"
down_revision = "a519c710877c"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "entity", sa.Column("nb_frames", sa.Integer(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("entity", "nb_frames")
    # ### end Alembic commands ###
