"""Add start and end dates to projects

Revision ID: 003be8a91001
Revises: 590aa1ffe731
Create Date: 2019-07-05 09:22:47.857496

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "003be8a91001"
down_revision = "590aa1ffe731"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("project", sa.Column("end_date", sa.Date(), nullable=True))
    op.add_column("project", sa.Column("start_date", sa.Date(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("project", "start_date")
    op.drop_column("project", "end_date")
    # ### end Alembic commands ###
