"""empty message

Revision ID: 818f7bda2528
Revises: bf1347acdee2
Create Date: 2018-04-10 22:45:19.462757

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "818f7bda2528"
down_revision = "bf1347acdee2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "preview_file", sa.Column("path", sa.String(length=400), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("preview_file", "path")
    # ### end Alembic commands ###
